@@ -1,6 +1,6 @@
 from sys import version_info
 from collections import deque
-from operator import itemgetter, attrgetter
+from operator import add, itemgetter, attrgetter
 from functools import partial
 from itertools import (islice, chain,                        
                        starmap, repeat, tee, cycle,
@@ -33,9 +33,6 @@ else:
     from itertools import filterfalse
     from itertools import zip_longest
 
-filterfalse = filterfalse
-zip_longest = zip_longest
-
 def take(limit, base): 
     return islice(base, limit)
 
@@ -205,3 +202,24 @@ def flatten(lists):
         else:
             yield l
 
+if version_info[0] == 3 and version_info[1] >= 3:
+    from itertools import accumulate
+else:
+    def accumulate(iterable, func=add):
+        """Make an iterator that returns accumulated sums. 
+        Elements may be any addable type including Decimal or Fraction. 
+        If the optional func argument is supplied, it should be a 
+        function of two arguments and it will be used instead of addition.
+
+        Origin implementation:
+        http://docs.python.org/dev/library/itertools.html#itertools.accumulate
+
+        Backported to work with all python versions (< 3.3)
+        """
+        it = iter(iterable)
+        total = next(it)
+        yield total
+        for element in it:
+            total = func(total, element)
+            yield total
+
