@@ -2,16 +2,15 @@
 
 """Tests for Fn.py library"""
 
+import sys
 import unittest
 import operator
 import itertools
 
-from sys import version_info
-
-from fn import op, _, F, Stream, iters, underscore, monad
+from fn import op, _, F, Stream, iters, underscore, monad, recur
 
 class InstanceChecker(object):
-    if version_info[0] == 2 and version_info[1] <= 6:
+    if sys.version_info[0] == 2 and sys.version_info[1] <= 6:
         def assertIsInstance(self, inst, cls):
             self.assertTrue(isinstance(inst, cls))
 
@@ -524,7 +523,10 @@ class OptionTestCase(unittest.TestCase, InstanceChecker):
         self.assertIsInstance(monad.Option(10), monad.Full)
         self.assertIsInstance(monad.Option(10, lambda x: x > 7), monad.Full)
         self.assertIsInstance(monad.Option(None), monad.Empty)
-        self.assertIsInstance(monad.Option(False), monad.Empty)
+        self.assertIsInstance(monad.Option(False), monad.Full)
+        self.assertIsInstance(monad.Option(0), monad.Full)
+        self.assertIsInstance(monad.Option(False, checker=bool), monad.Empty)
+        self.assertIsInstance(monad.Option(0, checker=bool), monad.Empty)
         self.assertIsInstance(monad.Option(10, lambda x: x > 70), monad.Empty)
 
     def test_map_filter(self):
@@ -539,21 +541,28 @@ class OptionTestCase(unittest.TestCase, InstanceChecker):
                                    .map(operator.methodcaller("strip"))
                                    .filter(len)
                                    .map(operator.methodcaller("upper"))
-                                   .getOr(""))
+                                   .get_or(""))
 
         # breaks on filter
         self.assertEqual("", r.parameter("empty")
                               .map(operator.methodcaller("strip"))
                               .filter(len)
                               .map(operator.methodcaller("upper"))
-                              .getOr(""))
+                              .get_or(""))
 
         # breaks on parameter
         self.assertEqual("", r.parameter("missed")
                               .map(operator.methodcaller("strip"))
                               .filter(len)
                               .map(operator.methodcaller("upper"))
-                              .getOr(""))
+                              .get_or(""))
+
+    def test_empty_check(self):
+        self.assertTrue(monad.Empty().empty)
+        self.assertTrue(monad.Option(None).empty)
+        self.assertTrue(monad.Option.from_call(lambda: None).empty)
+        self.assertFalse(monad.Option(10).empty)
+        self.assertFalse(monad.Full(10).empty)
 
     def test_lazy_orcall(self):
         def from_mimetype(request):
@@ -565,30 +574,29 @@ class OptionTestCase(unittest.TestCase, InstanceChecker):
             return monad.Option(request.get("url", None))\
                         .map(lambda s: s.split(".")[-1])
 
-
         # extract value from extension
         r = dict(url="myfile.png")
         self.assertEqual("PNG", monad.Option(r.get("type", None)) \
-                                     .orCall(from_mimetype, r) \
-                                     .orCall(from_extension, r) \
+                                     .or_call(from_mimetype, r) \
+                                     .or_call(from_extension, r) \
                                      .map(operator.methodcaller("upper")) \
-                                     .getOr(""))
+                                     .get_or(""))
 
         # extract value from mimetype
         r = dict(url="myfile.svg", mimetype="png")
         self.assertEqual("PNG", monad.Option(r.get("type", None)) \
-                                     .orCall(from_mimetype, r) \
-                                     .orCall(from_extension, r) \
+                                     .or_call(from_mimetype, r) \
+                                     .or_call(from_extension, r) \
                                      .map(operator.methodcaller("upper")) \
-                                     .getOr(""))
+                                     .get_or(""))
 
         # type is set directly
         r = dict(url="myfile.jpeg", mimetype="svg", type="png")
         self.assertEqual("PNG", monad.Option(r.get("type", None)) \
-                                     .orCall(from_mimetype, r) \
-                                     .orCall(from_extension, r) \
+                                     .or_call(from_mimetype, r) \
+                                     .or_call(from_extension, r) \
                                      .map(operator.methodcaller("upper")) \
-                                     .getOr(""))
+                                     .get_or(""))
 
     def test_optionable_decorator(self):
         class Request(dict):
@@ -603,7 +611,7 @@ class OptionTestCase(unittest.TestCase, InstanceChecker):
                                    .map(operator.methodcaller("strip"))
                                    .filter(len)
                                    .map(operator.methodcaller("upper"))
-                                   .getOr(""))
+                                   .get_or(""))
 
     def test_stringify(self):
         self.assertEqual("Full(10)", str(monad.Full(10)))
@@ -617,5 +625,41 @@ class OptionTestCase(unittest.TestCase, InstanceChecker):
         self.assertEqual("Empty()", repr(monad.Empty()))
         self.assertEqual("Empty()", repr(monad.Option(None)))
 
+    def test_static_constructor(self):
+        self.assertEqual(monad.Empty(),  monad.Option.from_value(None))
+        self.assertEqual(monad.Full(10), monad.Option.from_value(10))
+        self.assertEqual(monad.Empty(),  monad.Option.from_call(lambda: None))
+        self.assertEqual(monad.Full(10), monad.Option.from_call(operator.add, 8, 2))
+        self.assertEqual(monad.Empty(), 
+                         monad.Option.from_call(lambda d, k: d[k], 
+                                                {"a":1}, "b", exc=KeyError))
+
+class TrampolineTestCase(unittest.TestCase):
+
+    def test_tco_decorator(self):
+        def recur_accumulate(origin, f=operator.add, acc=0):
+            n = next(origin, None)
+            if n is None: return acc
+            return recur_accumulate(origin, f, f(acc, n))
+
+        # this works normally
+        self.assertEqual(10, recur_accumulate(iter(range(5))))
+
+        limit = sys.getrecursionlimit() * 10
+        # such count of recursive calls should fail on CPython,
+        # for PyPy we skip this test cause on PyPy the limit is
+        # approximative and checked at a lower level
+        if not hasattr(sys, 'pypy_version_info'):
+            self.assertRaises(RuntimeError, recur_accumulate, iter(range(limit)))
+
+        # with recur decorator it should run without problems
+        @recur.tco
+        def tco_accumulate(origin, f=operator.add, acc=0):
+            n = next(origin, None)
+            if n is None: return False, acc
+            return True, (origin, f, f(acc, n))
+
+        self.assertEqual(sum(range(limit)), tco_accumulate(iter(range(limit))))
+
 if __name__ == '__main__':
     unittest.main()
